from redis import StrictRedis

from .types import Hash

__all__ = ['Hash', 'get']


def get(**kwargs):
    return RedisStore(**kwargs)


class RedisStore:
    def __init__(self, **kwargs):
        self.redis_client = StrictRedis(**kwargs)

    def __repr__(self):
        connection_kwargs = self.redis_client.connection_pool.connection_kwargs
        return 'RedisStore(host={host!r}, port={port}, db={db})'.format(**connection_kwargs)

    def __getitem__(self, key):
        redis_type = self.redis_client.type(key).decode('utf-8')
        if redis_type == 'none':
            raise KeyError(key)

        if redis_type == 'string':
            value = self.redis_client.get(key)
            return value.decode()
        elif redis_type == 'hash':
            return Hash(key=key, store=self)

        raise NotImplementedError('{} type not supported'.format(redis_type))

    def __setitem__(self, key, value):
        if isinstance(value, dict):
            Hash(key=key, store=self, data=value)
        else:
            self.redis_client.set(key, value)

    def __delitem__(self, key):
        exists = self.redis_client.delete(key)
        if exists == 0:
            raise KeyError(key)

    def __contains__(self, key):
        return self.redis_client.exists(key) is True



import os
import re
import sys
from pathlib import Path
from shutil import rmtree

from setuptools import setup, find_packages, Command

here = Path.cwd()
with open(here / 'README.rst') as f:
    readme = '\n' + f.read()

with open(here / 'CHANGES.rst') as f:
    changes = '\n' + f.read()
    version_match = re.search(r'\n(\d+.\d+.\d+) /', changes)
    version = version_match.groups()[0]


class UploadCommand(Command):
    """Support setup.py publish."""

    description = 'Build and publish the package.'
    user_options = []

    @staticmethod
    def status(s):
        """Prints things in bold."""
        print('\033[1m{0}\033[0m'.format(s))

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        try:
            self.status('Removing previous builds…')
            rmtree(os.path.join(here, 'dist'))
        except FileNotFoundError:
            pass

        self.status('Building Source distribution…')
        os.system('{0} setup.py sdist'.format(sys.executable))

        self.status('Uploading the package to PyPi via Twine…')
        os.system('twine upload dist/*')

        self.status('Pushing git tags…')
        os.system('git tag v{0}'.format(version))
        os.system('git push --tags')

        sys.exit()


setup(
    name='redistore',
    version=version,
    description='Simple python interface for redis',
    url='https://github.com/lamenezes/redistore',
    author='Luiz Menezes',
    author_email='luiz.menezesf@gmail.com',
    license='MIT',
    long_description='\n' + readme,
    packages=find_packages(exclude=['tests*']),
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
    ],
    cmdclass={
        'upload': UploadCommand,
    },
)



from collections.abc import MutableMapping


class RedisType:
    def __init__(self, key=None, store=None, **kwargs):
        self._key = key
        self._store = store

    def __repr__(self):
        return '{}(key={!r})'.format(type(self).__name__, self._key)


class Hash(RedisType, MutableMapping):
    def __init__(self, key, store, data=None):
        super().__init__(key, store)

        self.hash_name = self._key
        if data:
            self.update(data)

    def __repr__(self):
        r = super().__repr__()[:-1]
        return '{}, data={!r})'.format(r, dict(self))

    def __getitem__(self, key):
        value = self._store.redis_client.hget(self.hash_name, key)
        if value is None:
            raise KeyError(key)
        return value.decode()

    def __setitem__(self, key, value):
        self._store.redis_client.hset(self.hash_name, key, value)

    def __delitem__(self, key):
        exists = self._store.redis_client.hdel(self.hash_name, key)
        if exists == 0:
            raise KeyError(key)

    def __iter__(self):
        return (key.decode() for key in self._store.redis_client.hkeys(self.hash_name))

    def __len__(self):
        return self._store.redis_client.hlen(self.hash_name)

    def __contains__(self, key):
        return self._store.redis_client.hexists(self.hash_name, key)

    def clear_keys(self, keys):
        keys = set(keys)
        inexistent_keys = keys - set(self.keys())
        if inexistent_keys:
            raise KeyError(inexistent_keys)

        for key in keys:
            del self[key]
